/*
 *  gtkmonop - the gtk+ monopoly simplator
 *  Portions copyright (C) 2001 Erik Bourget (ebourg@po-box.mcgill.ca)
 *  90% of this file is from Ka-Shu Wong's gtetrinet client, so
 *  Copyright (C) Ka-Shu Wong.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/* Thanks, gtetrinet. */

#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/time.h>
#include <sys/wait.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <errno.h>
#include <signal.h>
#include <ctype.h>
#include <malloc.h>
#include <libxml/tree.h>
#include <libxml/parser.h>
#include <libxml/xmlmemory.h>

#include "misc_functions.h"
#include "client.h"
#include "game_functions.h"
#include "stub.h"

#define MAXMESSAGE 20000 /* a big buffer */

int fdin[2], fdout[2]; /* two pipes, in and out */

char nick[1024];

int inputcbid;

int connected = 0;
char server[128];
int port;

extern int clientpid;

static int sock, connecterror;

/* structures and arrays for message translation */

struct inmsgt {
    enum inmsg_type num;
    char *str;
};

struct outmsgt {
    enum outmsg_type num;
    char *str;
};

#define IN_NOMSG 0;

struct inmsgt inmsgtable[] = {
    {IN_COMMANDLIST, "commandlist"},
    {IN_GAMELIST, "gamelist"},
    {IN_UPDATEGAMELIST, "updategamelist"},
    {IN_PLAYERUPDATE, "playerupdate"},
    {IN_UPDATEPLAYERLIST, "updateplayerlist"},
    {IN_JOINEDGAME, "joinedgame"},
    {IN_LEFTGAME, "leftgame"},
    {IN_MSG, "msg"},
    {IN_NEWTURN, "newturn"},
    {IN_ESTATEUPDATE, "estateupdate"},
    {IN_CONNECTED, "server"},
    {IN_DISCONNECTED, "disconnected"},
    {IN_CLIENT, "client"},
    {IN_TRADEUPDATE, "tradeupdate"},
    {IN_AUCTIONUPDATE, "auctionupdate"},
    /* internal communication here */

    {0, 0}
};

struct outmsgt outmsgtable[] = {
    {OUT_DISCONNECT, ".d"},
    {OUT_CONNECTED, "connected"},

    {OUT_CHAT, ""},
    
    {OUT_GAMENEW, ".gn"},
    {OUT_GAMEDESC, ".gd"},
    {OUT_GAMESTART, ".gs"},
    {OUT_BUY, ".eb"},
    {OUT_AUCTION, ".ea"},
    {OUT_AUCTIONBID, ".ab"},
    {OUT_BUYHOUSE, ".hb"},
    {OUT_SELLHOUSE, ".hs"},
    {OUT_ENDTURN, ".E"},
    {OUT_NICK, ".n"},
    {OUT_USECARD, ".c"},
    {OUT_LISTGAME, ".gl"},
    {OUT_JOINGAME, ".gj"},
    {OUT_LISTPLAYERS, ".gp"},
    {OUT_STARTGAME, ".gs"},
    {OUT_MORTGAGE, ".m"},
    {OUT_PAYJAIL, ".jp"},
    {OUT_JAILCARD, ".jc"},
    {OUT_JAILROLL, ".jr"},
    {OUT_TRADENEW, ".Tn"},
    {OUT_TRADEACCEPT, ".Ta"},
    {OUT_TRADEESTATE, ".Te"},
    {OUT_TRADEMONEY, ".Tm"},
    {OUT_TRADEREJECT, ".Tr"},
    {OUT_ROLL, ".r"},
    {OUT_UNMORTGAGE, ".u"},
    {OUT_CONFIRM, ".t"},
    

    {0, 0}
};

static void client_inputfunc (gpointer data, gint source,
                              GdkInputCondition condition);

/* functions which set up the connection */
static void client_process (void);
static void client_cleanpipe (void);
static int client_mainloop (void);
static int client_connect (void);
static void client_connected (void);
static int client_disconnect (void);

/* some other useful functions */
static int client_sendmsg (char *str);
int client_readmsg (char *str, int len);
int fdreadline (int fd, char *buf);

enum inmsg_type inmsg_translate (xmlNodePtr);
char *outmsg_translate (enum outmsg_type);

void client_initpipes (void)
{
    pipe (fdin);
    pipe (fdout);

    inputcbid = gdk_input_add (fdin[0], GDK_INPUT_READ,
                               (GdkInputFunction)client_inputfunc, NULL);
}

void client_destroypipes (void)
{
}

void client_init (char *s, int p)
{
    strcpy (server, s);
    port = p;
    
    if (clientpid) {
        if (connected) client_destroy ();
        else client_connectcancel ();
    }

    //    connectingdialog_new ();

    if ((clientpid = fork()) == 0) {
        client_process ();
        _exit (0);
    }

    client_connected();

}

void client_connectcancel (void)
{
    /* just kill the process */
    if (clientpid) kill (clientpid, SIGTERM);
    clientpid = 0;
}

void client_destroy (void)
{
	/* tell it to disconnect, then wait for it to die */
	if(clientpid) kill (clientpid, SIGTERM);
	waitpid (clientpid, NULL, 0);
	clientpid = 0;
}

/* a function that reads stuff from the pipe */
static void client_inputfunc (gpointer data, gint source,
                              GdkInputCondition condition)
{
    char *buf;
    enum inmsg_type msgtype;
    xmlDocPtr doc;
    xmlNodePtr node;

    buf = malloc(MAXMESSAGE);
    fdreadline (fdin[0], buf);

    fprintf(stderr, "%s\n", buf);

    if(buf[0] == '<') { /* looks like an XML element */
	    doc = xmlParseMemory(buf, strlen(buf));
	    if((doc != (xmlDocPtr)NULL) &&
	       (!xmlStrcmp("monopd", doc->children->name))) {
		    node = doc->children->children; /* after the <monopd> */
		    do {
			    msgtype = inmsg_translate(node);
			    monopd_inmessage(msgtype, doc, node, sock);
			    node = node->next;
		    } while(node != NULL); /* get all messages */ 
	    }
	    
    } else {
	    /* nothing */    
    }
    
    free(buf);
}

void client_outmessage (enum outmsg_type msgtype, char *str)
{
    char *buf;
    buf = malloc(MAXMESSAGE);
    
    strcpy (buf, outmsg_translate (msgtype));
    if (str) {
        strcat (buf, str);
    }

    sprintf(buf, "%s%c", buf, 13);
    
    write (fdout[1], buf, strlen(buf));
    write (fdout[1], "\n", 1);
    free(buf);
}

void client_inmessage (char *str)
{
    write (fdin[1], str, strlen(str));
    write (fdin[1], "\n", 1);
}

/* these functions set up the connection */

void client_process (void)
{
    errno = 0;
    if (client_connect () == -1) {
        char errmsg[MAXMESSAGE];

        strcpy (errmsg, "noconnecting ");

        if (errno) strcat (errmsg, strerror (errno));
        else if (h_errno) strcat (errmsg, "Unknown host");

        client_inmessage (errmsg);

        return;
    }
    
    client_connected();
    
    connecterror = 0;

    client_cleanpipe ();
    client_mainloop ();
    client_disconnect ();
    return;
}

void client_cleanpipe ()
{
    fd_set rfds;
    struct timeval tv;
    char buf[MAXMESSAGE];

    while (1) {
        FD_ZERO (&rfds);
        FD_SET (fdout[0], &rfds);
        tv.tv_sec = 0; tv.tv_usec = 0;
        if (select (fdout[0]+1, &rfds, NULL, NULL, &tv)) fdreadline (fdout[0], buf);
        else return;
    }
}

int client_mainloop (void)
{
    fd_set rfds;
    struct timeval tv;
    int m;

    /* read in stuff from the socket and the pipe */
    while (1) {
        FD_ZERO (&rfds);
        FD_SET (sock, &rfds);
        FD_SET (fdout[0], &rfds);

        m = sock > fdout[0] ? sock : fdout[0];
        tv.tv_sec = 0;
        tv.tv_usec = 50000; /* wait a little while */
        if (select (m+1, &rfds, NULL, NULL, &tv))
        {
            char buf[MAXMESSAGE]; /* a big buffer */

            /* we have something to read */

            /* is it the socket? */
            FD_ZERO (&rfds); FD_SET (sock, &rfds);
            tv.tv_sec = 0; tv.tv_usec = 0;
            if (select (sock+1, &rfds, NULL, NULL, &tv)) {
		    if (client_readmsg (buf, sizeof(buf)) < 0) {
			    return -1;
		    }
		    
		    client_inmessage (buf); /* send to parent process */
            }

            /* or it the pipe? */
            FD_ZERO (&rfds); FD_SET (fdout[0], &rfds);
            tv.tv_sec = 0; tv.tv_usec = 0;
            if (select (fdout[0]+1, &rfds, NULL, NULL, &tv)) {
		    fdreadline (fdout[0], buf);
		    if (strcmp (buf, outmsg_translate(IN_DISCONNECTED)) == 0)
			    goto clientend;
		    else if (strcmp (buf, outmsg_translate(IN_CONNECTED)) == 0)
			    client_connected ();
		    else
			    client_sendmsg (buf);
            }
        }
    }
clientend:
    return 0;
}

int client_connect (void)
{
    struct hostent *h;
    struct sockaddr_in sa;

    /* set up the connection */

    h = gethostbyname (server);
    if (h == 0) return -1;
    memset (&sa, 0, sizeof (sa));
    memcpy (&sa.sin_addr, h->h_addr, h->h_length);
    sa.sin_family = h->h_addrtype;
    
    sa.sin_port = htons (port);

    sock = socket (sa.sin_family, SOCK_STREAM, 0);
    if (sock < 0) return -1;

    if (connect (sock, (struct sockaddr *)&sa, sizeof(sa)) < 0)
        return -1;

    return 0;
}

void client_connected (void)
{
	connected = 1;
	//	client_inmessage("connect");
}

int client_disconnect (void)
{
    shutdown (sock, 2);
    close (sock);

    //    if (!connecterror || connected) client_inmessage (IN_DISCONNECTED);

    return 0;
}


/* some other useful functions */

int client_sendmsg (char *str)
{
    char c = 0xFF;
    
    fprintf(stderr, "out: %s\n", str);
    
    if (write (sock, str, strlen (str))==-1) return -1;
    if (write (sock, &c, 1)==-1) return -1;

    return 0;
}

int client_readmsg (char *str, int len)
{
    int i = 0;

    /* read it in one char at a time */
    for (;i < len-1; i++) {
	    
	    if (read (sock, &str[i], 1) != 1)
		    /* we have an error */
		    return -1;
	    if (str[i]=='\n') break;
    }
    str[i] = 0;
    return i;
}

enum inmsg_type inmsg_translate (xmlNodePtr node)
{
	char *str;
	int i;

	str = (char *)(node->name);
		
	for (i = 0; inmsgtable[i].str; i++) {
		if(!strcmp(str, inmsgtable[i].str)) {
			return inmsgtable[i].num;
		}
	}
	return 0;
}

char *outmsg_translate (enum outmsg_type num)
{
    int i;
    for (i = 0; outmsgtable[i].num; i++) {
        if (outmsgtable[i].num==num) return outmsgtable[i].str;
    }
    return NULL;
}

int fdreadline (int fd, char *buf)
{
    int c = 0;
    /* read a single line - no overflow check */
    while (1) {
        if (read (fd, &buf[c], 1) == 0) break;
        if (buf[c] == '\n') break;
        c ++;
    }
    buf[c++] = 0;
    return c;
}

void shutdown_gtkmonop(void)
{
	extern int clientpid;
	
	/* close down the network connection */
	client_destroy();
	kill(clientpid, SIGTERM);
}
